/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include "syntax.h"

#include <QColor>
#include <QJsonArray>
#include <QJsonDocument>
#include <QJsonObject>
#include <QMessageBox>

Syntax::Syntax(QTextDocument *document)
   : QSyntaxHighlighter(document)
{
}

bool Syntax::processSyntax()
{
   QByteArray data = readJson("../syn_cpp.json");

   QJsonDocument doc  = QJsonDocument::fromJson(data);
   QJsonObject object = doc.object();
   QJsonArray list;

   HighlightingRule rule;

   // keywords
   list = object.value("keywords").toArray();

   rule.format.setFontItalic(false);
   rule.format.setForeground(QColor(0,0,255));

   for (const auto &item : list) {
      rule.pattern = QRegularExpression(item.toString());
      highlightingRules.append(rule);
   }

   // classes
   list = object.value("classes").toArray();

   rule.format.setFontItalic(false);
   rule.format.setForeground(QColor(165,0,165));

   for (const auto &item : list) {
      rule.pattern = QRegularExpression(item.toString());
      highlightingRules.append(rule);
   }

   // functions
   list = object.value("functions").toArray();

   rule.format.setFontItalic(false);
   rule.format.setForeground(QColor(0,128,255));

   for (const auto &item : list) {
      rule.pattern = QRegularExpression(item.toString());
      highlightingRules.append(rule);
   }

   // types
   list = object.value("types").toArray();

   rule.format.setFontItalic(false);
   rule.format.setForeground(QColor(162,82,0));

   for (const auto &item : list) {
      rule.pattern = QRegularExpression(item.toString());
      highlightingRules.append(rule);
   }

   // quoted text
   rule.format.setFontItalic(false);
   rule.format.setForeground(QColor(140,140,0));

   rule.pattern = QRegularExpression("\".*?\"");
   highlightingRules.append(rule);

   // single line comment
   QString commentSingle = object.value("comment-single").toString();

   rule.format.setFontItalic(true);
   rule.format.setForeground(QColor(0,128,0));
   rule.pattern = QRegularExpression(commentSingle);
   highlightingRules.append(rule);

   rehighlight();

   return true;
}

QByteArray Syntax::readJson(QString fName)
{
   QByteArray data;

   QFile file(fName);
   if (! file.open(QFile::ReadOnly | QFile::Text)) {
      const QString msg = "Unable to open Syntax file: " + fName + " : " + file.errorString();
      QMessageBox::critical(nullptr, "Open File", msg, QMessageBox::Cancel);

      return data;
   }

   data = file.readAll();
   file.close();

   return data;
}

void Syntax::highlightBlock(const QString &text)
{
   QRegularExpressionMatch match;

   for (auto &rule : highlightingRules) {
      match = rule.pattern.match(text);

      while (match.hasMatch()) {
         int index  = match.capturedStart(0) - text.begin();
         int length = match.capturedLength();

         setFormat(index, length, rule.format);

         // get new match
         match = rule.pattern.match(text, match.capturedEnd(0));
      }
   }
}
